@@ -1,10 +1,12 @@
 """
-This module uses the GDAL and OGR Library to read maps in vaious formats and
+This module uses the GDAL and OGR Library to read maps in various formats and
 export the results of Epigrass simulations to the formats supported by these libraries
 
 copyright 2007,2012 by Flavio Codeco Coelho
 Licensed under the GPL.
 """
+from __future__ import absolute_import
+from __future__ import print_function
 import locale, os, pylab
 from osgeo import ogr, osr, gdal
 from xml.dom import minidom, Node
@@ -12,8 +14,26 @@ from matplotlib.colors import rgb2hex, LogNorm
 from matplotlib.colors import Normalize
 from matplotlib import cm
 from numpy import array
+import geopandas as gpd
 from zipfile import ZipFile
 import json
+import six
+from six.moves import range
+from six.moves import zip
+
+class NewWorld:
+    def __init__(self, filename, namefield, geocfield, outdir='.'):
+        """
+        Open Shapefile, and load it into a geopandas dataframe
+        :param filename:
+        :param namefield: name of the field containing locality name
+        :param geocfield: name of the field containing geocode
+        :param outdir: directory for data output
+        """
+        self.geocfield = geocfield
+        self.namefield = namefield
+        self.outdir = outdir
+        self.map = gpd.read_file(filename)
 
 
 class World:
@@ -21,6 +41,8 @@ class World:
         '''
         Instantiate a world object.
         Filename points to a file supported by OGR
+        namefield - name of the field containing polygon name
+        geocfield - name of the field containing geocode
         '''
         self.geocfield = geocfield
         self.namefield = namefield
@@ -60,7 +82,7 @@ class World:
         """
         nlay = self.ds.GetLayerCount()
         ln = []
-        for i in xrange(nlay):
+        for i in range(nlay):
             l = self.ds.GetLayer(i)
             ln.append(l.GetName())
         return ln
@@ -110,7 +132,7 @@ class World:
                 self.centdict[f.GetFieldAsInteger(self.geocfield)] = (c.GetX(), c.GetY(), c.GetZ())
                 self.namedict[f.GetFieldAsInteger(self.geocfield)] = f.GetField(self.namefield)
             except:  # in case feature is not a polygon
-                print  f.GetFID(), g.GetGeometryType()
+                print(f.GetFID(), g.GetGeometryType())
             f = l.GetNextFeature()
             # print (2600501 in self.centdict)
 
@@ -120,8 +142,6 @@ class World:
         The node layer will be based on the centroids of the
         polygons belonging to the map layer associated with this
         world instance.
-        namefield - name of the field containing polygon name
-        geocfield - name of the field containing geocode
         """
         # Creates a new shape file to hold the data
         if os.path.exists(os.path.join(self.outdir, 'Nodes.shp')):
@@ -147,8 +167,14 @@ class World:
             x = self.centdict[gc][0]
             y = self.centdict[gc][1]
             fe = ogr.Feature(nl.GetLayerDefn())
-            name = f.GetField(self.namefield)
-            fe.SetField('name', f.GetField(self.namefield))
+            # name =f.GetField(self.namefield).decode('utf8', errors="surrogateescape")
+            # print(type(self.namefield))
+            try:
+                fe.SetField('name', f.GetField(self.namefield).decode('utf8', "surrogateescape"))
+            except AttributeError:
+                fe.SetField('name', '')
+            except UnicodeEncodeError:
+                fe.SetField('name', '')
             fe.SetField('geocode', gc)
             fe.SetField('x', str(x))
             fe.SetField('y', str(y))
@@ -252,7 +278,7 @@ class World:
             try:
                 geom = self.geomdict[gc]
             except KeyError:  # Geocode not in polygon dictionary
-                raise KeyError("Geocode %s not in polygon dictionary\n{}" % gc)
+                raise KeyError("Geocode %s not in polygon dictionary\n" % gc)
             if geom.GetGeometryType() != 3: continue
             # print geom.GetGeometryCount()
             fe = ogr.Feature(dl.GetLayerDefn())
@@ -291,7 +317,7 @@ class World:
         :Parameters:
         :parameter dl: datalayer to save
         """
-        print "==> saving to GeoJSON"
+        print("==> saving to GeoJSON")
         spatial_reference = dl.GetSpatialRef()
 
         feature_collection = {"type": "FeatureCollection",
@@ -439,7 +465,7 @@ class AnimatedKML(object):
             ts.appendChild(w)
             pm_newtime.appendChild(ts)
             self.doc.appendChild(pm_newtime)
-        for pm in self.pmdict.itervalues():
+        for pm in six.itervalues(self.pmdict):
             self.doc.removeChild(pm)
         self.pmdict = {}
 
@@ -563,7 +589,7 @@ class KmlGenerator:
                     try:
                         name = str(geoc) + "-" + names[geoc]
                     except KeyError:
-                        print geoc
+                        print(geoc)
                         name = ""
                 description = "Prevalence: %s;\nTotal cases: %s;\nImported Cases: %s;" % (
                     prevalence, f.GetField("totalcases"), f.GetField("arrivals"))
@@ -628,9 +654,9 @@ def gdal_error_handler(err_class, err_num, err_msg):
     }
     err_msg = err_msg.replace('\n', ' ')
     err_class = errtype.get(err_class, 'None')
-    print 'Error Number: %s' % (err_num)
-    print 'Error Type: %s' % (err_class)
-    print 'Error Message: %s' % (err_msg)
+    print('Error Number: %s' % (err_num))
+    print('Error Type: %s' % (err_class))
+    print('Error Message: %s' % (err_msg))
 
 
 # install error handler
