@@ -1,6 +1,3 @@
-# coding:utf-8
-from __future__ import division
-
 """
 Library of discrete time Epidemic models
 
@@ -49,10 +46,11 @@ class Epimodel(object):
     """
     Defines a library of discrete time population models
     """
+
     @cython.locals(geocode='long', modtype='bytes', parallel='bint')
     def __init__(self, geocode, modtype=b'', parallel=True):
         """
-        defines which models a given site will use
+        Defines which models a given site will use
         and set variable names accordingly.
         :param modtype:
         """
@@ -62,17 +60,18 @@ class Epimodel(object):
 
     def __call__(self, *args, **kwargs):
         args = self.get_args_from_redis()
-        res = self.step(*args)
+        res = self.step(*(list(args)+[self]))
         self.update_redis(res)
         # return res
 
-    @cython.locals( simstep='long', totpop='long', theta='double', npass='double')
+    @cython.locals(simstep='long', totpop='long', theta='double', npass='double')
     def get_args_from_redis(self):
         """
-        get updated parameters from the redis database.
-        :param geocode: geocode of the site running this model.
+        Get updated parameters from the redis database.
         """
-        inits = [int(nan_to_num(i)) for i in eval(redisclient.lindex("{}:inits".format(self.geocode), -1))]
+        sinits = redisclient.lrange("{}:inits".format(self.geocode),-1,-1)
+        # print(sinits)
+        inits = eval(sinits[0])
         simstep = int(redisclient.get("simstep"))
         totpop = int(float(redisclient.get("{}:totpop".format(self.geocode))))
         theta = int(nan_to_num(float(redisclient.get("{}:theta".format(self.geocode)))))
@@ -91,8 +90,9 @@ class Epimodel(object):
         """
         # Site state
         state, Lpos, migInf = results
-        redisclient.rpush("{}:inits".format(self.geocode), state)  # updating inits
-        redisclient.rpush('{}:ts'.format(self.geocode), state)
+        # print("Updating redis: ", state, migInf)
+        redisclient.rpush("{}:inits".format(self.geocode), str(state))  # updating inits
+        redisclient.rpush('{}:ts'.format(self.geocode), str(state))
         redisclient.set('{}:Lpos'.format(self.geocode), Lpos)
         totc = int(nan_to_num(float(redisclient.get('{}:totalcases'.format(self.geocode)))))
         redisclient.set('{}:totalcases'.format(self.geocode), Lpos + totc)
@@ -102,54 +102,57 @@ class Epimodel(object):
         # Graph state
         if Lpos > 0:
             infected = int(redisclient.get("simstep"))
-            redisclient.rpush("epipath", (infected, self.geocode, {}))  # TODO: replace empty dict with infectors
+            redisclient.rpush("epipath", str((infected, self.geocode, {})))  # TODO: replace empty dict with infectors
             # self.parentGraph.epipath.append((self.parentGraph.simstep, self.geocode, self.infector))
             # TODO: have infector be stated in terms of geocodes
 
+
 @cython.locals(Type='bytes')
-def selectModel(Type):
+def selectModel(modtype):
     """
     Sets the model engine
     """
-    if Type == 'SIR':
+    if isinstance(modtype, str):
+        modtype = bytes(modtype, 'utf8')
+    if modtype == b'SIR':
         return stepSIR
-    elif Type == 'SIR_s':
+    elif modtype == b'SIR_s':
         return stepSIR_s
-    elif Type == 'SIS':
+    elif modtype == b'SIS':
         return stepSIS
-    elif Type == 'SIS_s':
+    elif modtype == b'SIS_s':
         return stepSIS_s
-    elif Type == 'SEIS':
+    elif modtype == b'SEIS':
         return stepSEIS
-    elif Type == 'SEIS_s':
+    elif modtype == b'SEIS_s':
         return stepSEIS_s
-    elif Type == 'SEIR':
+    elif modtype == b'SEIR':
         return stepSEIR
-    elif Type == 'SEIR_s':
+    elif modtype == b'SEIR_s':
         return stepSEIR_s
-    elif Type == 'SIpRpS':
+    elif modtype == b'SIpRpS':
         return stepSIpRpS
-    elif Type == 'SIpRpS_s':
+    elif modtype == b'SIpRpS_s':
         return stepSIpRpS_s
-    elif Type == 'SEIpRpS':
+    elif modtype == b'SEIpRpS':
         return stepSEIpRpS
-    elif Type == 'SEIpRpS_s':
+    elif modtype == b'SEIpRpS_s':
         return stepSEIpRpS_s
-    elif Type == 'SIpR':
+    elif modtype == b'SIpR':
         return stepSIpR
-    elif Type == 'SIpR_s':
+    elif modtype == b'SIpR_s':
         return stepSIpR_s
-    elif Type == 'SEIpR':
+    elif modtype == b'SEIpR':
         return stepSEIpR
-    elif Type == 'SEIpR_s':
+    elif modtype == b'SEIpR_s':
         return stepSEIpR_s
-    elif Type == 'SIRS':
+    elif modtype == b'SIRS':
         return stepSIRS
-    elif Type == 'SIRS_s':
+    elif modtype == b'SIRS_s':
         return stepSIRS_s
-    elif Type == 'Influenza':
+    elif modtype == b'Influenza':
         return stepFlu
-    elif Type == 'Custom':
+    elif modtype == b'Custom':
         # adds the user model as a method of instance self
         try:
             # TODO: move this import to the graph level
@@ -157,7 +160,7 @@ def selectModel(Type):
 
             return CustomModel.Model
         except ImportError:
-            print "You have to Create a CustomModel.py file before you can select\nthe Custom model type"
+            print("You have to Create a CustomModel.py file before you can select\nthe Custom model type")
     else:
         sys.exit('Model type specified in .epg file is invalid')
 
@@ -166,7 +169,7 @@ def selectModel(Type):
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepFlu(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepFlu(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     Flu model with classes S,E,I subclinical, I mild, I medium, I serious, deaths
     """
@@ -176,32 +179,32 @@ def stepFlu(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=N
               'Susc_age3', 'Incub_age3', 'Subc_age3', 'Sympt_age3', 'Comp_age3',
               'Susc_age4', 'Incub_age4', 'Subc_age4', 'Sympt_age4', 'Comp_age4',)
     if simstep == 1:  # get initial values
-        S1, E1, Is1, Ic1, Ig1 = (bi['susc_age1'], bi['incub_age1'], bi['subc_age1'], bi['sympt_age1'], bi['comp_age1'])
-        S2, E2, Is2, Ic2, Ig2 = (bi['susc_age2'], bi['incub_age2'], bi['subc_age2'], bi['sympt_age2'], bi['comp_age2'])
-        S3, E3, Is3, Ic3, Ig3 = (bi['susc_age3'], bi['incub_age3'], bi['subc_age3'], bi['sympt_age3'], bi['comp_age3'])
-        S4, E4, Is4, Ic4, Ig4 = (bi['susc_age4'], bi['incub_age4'], bi['subc_age4'], bi['sympt_age4'], bi['comp_age4'])
+        S1, E1, Is1, Ic1, Ig1 = (bi[b'susc_age1'], bi[b'incub_age1'], bi[b'subc_age1'], bi[b'sympt_age1'], bi[b'comp_age1'])
+        S2, E2, Is2, Ic2, Ig2 = (bi[b'susc_age2'], bi[b'incub_age2'], bi[b'subc_age2'], bi[b'sympt_age2'], bi[b'comp_age2'])
+        S3, E3, Is3, Ic3, Ig3 = (bi[b'susc_age3'], bi[b'incub_age3'], bi[b'subc_age3'], bi[b'sympt_age3'], bi[b'comp_age3'])
+        S4, E4, Is4, Ic4, Ig4 = (bi[b'susc_age4'], bi[b'incub_age4'], bi[b'subc_age4'], bi[b'sympt_age4'], bi[b'comp_age4'])
     else:  # get values from last time step
         S1, E1, Is1, Ic1, Ig1, S2, E2, Is2, Ic2, Ig2, S3, E3, Is3, Ic3, Ig3, S4, E4, Is4, Ic4, Ig4 = inits
     N = totpop
 
     # for k, v in bp.items():
     #     exec ('%s = %s' % (k, v))
-    alpha = bp['alpha']
-    beta = bp['beta']
-    r = bp['r']
-    e = bp['e']
-    c = bp['c']
-    g = bp['g']
-    d = bp['d']
-    pc1 = bp['pc1']
-    pc2 = bp['pc2']
-    pc3 = bp['pc3']
-    pc4 = bp['pc4']
-    pp1 = bp['pp1']
-    pp2 = bp['pp2']
-    pp3 = bp['pp3']
-    pp4 = bp['pp4']
-    b = bp['b']
+    alpha = bp[b'alpha']
+    beta = bp[b'beta']
+    r = bp[b'r']
+    e = bp[b'e']
+    c = bp[b'c']
+    g = bp[b'g']
+    d = bp[b'd']
+    pc1 = bp[b'pc1']
+    pc2 = bp[b'pc2']
+    pc3 = bp[b'pc3']
+    pc4 = bp[b'pc4']
+    pp1 = bp[b'pp1']
+    pp2 = bp[b'pp2']
+    pp3 = bp[b'pp3']
+    pp4 = bp[b'pp4']
+    b = bp[b'b']
 
     # Vacination event
 
@@ -252,7 +255,7 @@ def stepFlu(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=N
 
     # Migrating infecctious
     migInf = (
-        Ig1pos + Ig2pos + Ig3pos + Ig4pos + Ic1pos + Ic2pos + Ic3pos + Ic4pos + 0.5 * (
+            Ig1pos + Ig2pos + Ig3pos + Ig4pos + Ic1pos + Ic2pos + Ic3pos + Ic4pos + 0.5 * (
             Is1pos + Is2pos + Is3pos + Is4pos))
     # Return variable values
 
@@ -260,11 +263,12 @@ def stepFlu(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=N
             Ic2pos, Ig2pos, S3pos, E3pos, Is3pos, Ic3pos, Ig3pos, S4pos,
             E4pos, Is4pos, Ic4pos, Ig4pos], Lpos, migInf
 
-@cython.locals(inits= 'object', simstep='long', totpop='long', theta='double', npass='double',
+
+@cython.locals(inits='object', simstep='long', totpop='long', theta='double', npass='double',
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SIS, and return its values (no demographics)
     - inits = (E,I,S)
@@ -304,7 +308,7 @@ def stepSIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=N
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SIS:
     - inits = (E,I,S)
@@ -346,23 +350,24 @@ def stepSIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SIR, and return its values (no demographics)
     - inits = (E,I,S)
     - theta = infectious individuals from neighbor sites
     """
+    # print(inits)
     if simstep == 1:  # get initial values
-        E, I, S = (bi['e'], bi['i'], bi['s'])
+        E, I, S = (bi.get(b'e', 0), bi[b'i'], bi[b's'])
     else:
         E, I, S = inits
     N = totpop
-    beta = bp['beta'];
-    alpha = bp['alpha'];
+    beta = bp[b'beta'];
+    alpha = bp[b'alpha'];
     # e = bp['e'];
-    r = bp['r'];
+    r = bp[b'r'];
     # delta = bp['delta'];
-    b = bp['b'];
+    b = bp[b'b'];
     # w = bp['w'];
     # p = bp['p']
     Lpos = float(beta) * S * ((I + theta) / (N + npass)) ** alpha  # Number of new cases
@@ -375,14 +380,14 @@ def stepSIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=N
     # Migrating infecctious
     migInf = Ipos
 
-    return [0, Ipos, Spos], Lpos, migInf
+    return (0, Ipos, Spos), Lpos, migInf
 
 
 @cython.locals(inits='object', simstep='long', totpop='long', theta='double', npass='double',
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SIR:
     - inits = (E,I,S)
@@ -424,7 +429,7 @@ def stepSIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSEIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     Defines the model SEIS:
     - inits = (E,I,S)
@@ -460,7 +465,7 @@ def stepSEIS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSEIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SEIS:
     - inits = (E,I,S)
@@ -502,7 +507,7 @@ def stepSEIS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, value
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSEIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     Defines the model SEIR:
     - inits = (E,I,S)
@@ -540,7 +545,7 @@ def stepSEIR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSEIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SEIR:
     - inits = (E,I,S)
@@ -585,7 +590,7 @@ def stepSEIR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, value
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SIpRpS, and return its values (no demographics)
     - inits = (E,I,S)
@@ -621,7 +626,7 @@ def stepSIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, value
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SIpRpS:
     - inits = (E,I,S)
@@ -663,7 +668,7 @@ def stepSIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, val
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSEIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     Defines the model SEIpRpS:
     - inits = (E,I,S)
@@ -700,7 +705,7 @@ def stepSEIpRpS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, valu
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSEIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SEIpRpS:
     - inits = (E,I,S)
@@ -742,7 +747,7 @@ def stepSEIpRpS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, va
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SIpR, and return its values (no demographics)
     - inits = (E,I,S)
@@ -780,7 +785,7 @@ def stepSIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SIpRs:
     - inits = (E,I,S)
@@ -827,7 +832,7 @@ def stepSIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, value
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSEIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SEIpR, and return its values (no demographics)
     - inits = (E,I,S)
@@ -867,7 +872,7 @@ def stepSEIpR(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSEIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSEIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SEIpRs:
     - inits = (E,I,S)
@@ -916,7 +921,7 @@ def stepSEIpR_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, valu
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIRS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None):
+def stepSIRS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None):
     """
     calculates the model SIRS, and return its values (no demographics)
     - inits = (E,I,S)
@@ -953,7 +958,7 @@ def stepSIRS(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=
                beta='double', alpha='double', E='double', I='double', S='double', N='long',
                r='double', b='double', w='double', Lpos='double', Lpos_esp='double', R='double',
                Ipos='double', Spos='double', Rpos='double')
-def stepSIRS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, dist='poisson'):
+def stepSIRS_s(inits, simstep, totpop, theta=0, npass=0, bi=None, bp=None, values=None, model=None, dist='poisson'):
     """
     Defines an stochastic model SIR:
     - inits = (E,I,S)
